@@ -282,10 +282,22 @@ void
 cb_mini_tweet_parse (CbMiniTweet *t,
                      JsonObject  *obj)
 {
-  GDateTime *time = parse_created_at (json_object_get_string_member (obj, "created_at"));
+  GDateTime *time;
+  JsonObject *extended_object;
+
+  if (json_object_has_member (obj, "extended_tweet"))
+    extended_object = json_object_get_object_member (obj, "extended_tweet");
+  else
+    extended_object = obj;
+
+  time = parse_created_at (json_object_get_string_member (obj, "created_at"));
 
   t->id = json_object_get_int_member (obj, "id");
-  t->text = g_strdup (json_object_get_string_member (obj, "text"));
+  if (json_object_has_member (extended_object, "full_text"))
+    t->text = g_strdup (json_object_get_string_member (extended_object, "full_text"));
+  else
+    t->text = g_strdup (json_object_get_string_member (extended_object, "text"));
+
   t->created_at = g_date_time_to_unix (time);
   cb_user_identity_parse (&t->author, json_object_get_object_member (obj, "user"));
 
@@ -306,17 +318,28 @@ void
 cb_mini_tweet_parse_entities (CbMiniTweet *t,
                               JsonObject  *status)
 {
-  JsonObject *entities     = json_object_get_object_member (status, "entities");
-  JsonArray *urls          = json_object_get_array_member (entities, "urls");
-  JsonArray *hashtags      = json_object_get_array_member (entities, "hashtags");
-  JsonArray *user_mentions = json_object_get_array_member (entities, "user_mentions");
+  JsonObject *extended_obj = status;
+  JsonObject *entities;
+  JsonArray *urls;
+  JsonArray *hashtags;
+  JsonArray *user_mentions;
   JsonArray *media_arrays[2];
-  int media_count = json_object_get_member_size (entities, "media");
+  int media_count;
   guint i, p;
   int url_index = 0;
   guint n_media_arrays = 0;
   int max_entities;
 
+  if (json_object_has_member (status, "extended_tweet"))
+    extended_obj = json_object_get_object_member (status, "extended_tweet");
+
+  entities      = json_object_get_object_member (extended_obj, "entities");
+  urls          = json_object_get_array_member (entities, "urls");
+  hashtags      = json_object_get_array_member (entities, "hashtags");
+  user_mentions = json_object_get_array_member (entities, "user_mentions");
+  media_count   = json_object_get_member_size (entities, "media");
+
+
   if (json_object_has_member (status, "extended_entities"))
     media_count +=  json_object_get_member_size (json_object_get_object_member (status, "extended_entities"),
                                                  "media");
@@ -387,7 +410,7 @@ cb_mini_tweet_parse_entities (CbMiniTweet *t,
       t->entities[url_index].from = json_array_get_int_element (indices, 0);
       t->entities[url_index].to   = json_array_get_int_element (indices, 1);
       t->entities[url_index].display_text = g_strdup_printf ("@%s", screen_name);
-      t->entities[url_index].tooltip_text = g_strdup (json_object_get_string_member (mention, "name"));
+      t->entities[url_index].tooltip_text = escape_ampersand (json_object_get_string_member (mention, "name"));
       t->entities[url_index].target = g_strdup_printf ("@%s/@%s", id_str, screen_name);
       url_index ++;
     }
@@ -401,12 +424,28 @@ cb_mini_tweet_parse_entities (CbMiniTweet *t,
         {
           JsonObject *url = json_node_get_object (json_array_get_element (medias, i));
           JsonArray  *indices = json_object_get_array_member (url, "indices");
-          /*const char *expanded_url = json_object_get_string_member (url, "expanded_url");*/
+          char *url_str = escape_ampersand (json_object_get_string_member (url, "url"));
+          int k;
+          gboolean duplicate = FALSE;
+
+          /* Check for duplicates */
+          for (k = 0; k < url_index; k ++)
+            {
+              const char *target = t->entities[k].target;
+              if (target != NULL && strcmp (target, url_str) == 0)
+                {
+                  duplicate = TRUE;
+                  break;
+                }
+            }
+
+          if (duplicate)
+            continue;
 
           t->entities[url_index].from = json_array_get_int_element (indices, 0);
           t->entities[url_index].to   = json_array_get_int_element (indices, 1);
           t->entities[url_index].display_text = escape_ampersand (json_object_get_string_member (url, "display_url"));
-          t->entities[url_index].target = escape_ampersand (json_object_get_string_member (url, "url"));
+          t->entities[url_index].target = url_str;
 
           url_index ++;
         }
